@@ -1505,6 +1505,8 @@ static int proc_meminfo_read(char *buf, size_t size, off_t offset,
 		total_len += l;
 	}
 
+	fclose(f);
+	free(line);
 	return total_len;
 }
 
@@ -1631,6 +1633,8 @@ static int proc_cpuinfo_read(char *buf, size_t size, off_t offset,
 		}
 	}
 
+	fclose(f);
+	free(line);
 	return total_len;
 }
 
@@ -1642,7 +1646,7 @@ static int proc_stat_read(char *buf, size_t size, off_t offset,
 	nih_local char *cpuset = NULL;
 	char *line = NULL;
 	size_t linelen = 0, total_len = 0;
-	int curcpu = 0;
+	int curcpu = -1; /* cpu numbering starts at 0 */
 	FILE *f;
 
 	if (offset)
@@ -1662,16 +1666,20 @@ static int proc_stat_read(char *buf, size_t size, off_t offset,
 	while (getline(&line, &linelen, f) != -1) {
 		size_t l;
 		int cpu;
+		char cpu_char[10]; /* That's a lot of cores */
 		char *c;
 
-		if (sscanf(line, "cpu%d", &cpu) != 1) {
-			/* not a ^cpu line, just print it */
+		if (sscanf(line, "cpu%9[^ ]", cpu_char) != 1) {
+			/* not a ^cpuN line containing a number N, just print it */
 			l = snprintf(buf, size, "%s", line);
 			buf += l;
 			size -= l;
 			total_len += l;
 			continue;
 		}
+
+		if (sscanf(cpu_char, "%d", &cpu) != 1)
+			continue;
 		if (!cpu_in_cpuset(cpu, cpuset))
 			continue;
 		curcpu ++;
@@ -1685,6 +1693,8 @@ static int proc_stat_read(char *buf, size_t size, off_t offset,
 		total_len += l;
 	}
 
+	fclose(f);
+	free(line);
 	return total_len;
 }
 
@@ -1808,9 +1818,12 @@ static long int getprocidle(void)
 {
 	FILE *f = fopen("/proc/uptime", "r");
 	long int age, idle;
+	int ret;
 	if (!f)
 		return 0;
-	if (fscanf(f, "%ld %ld", &age, &idle) != 2)
+	ret = fscanf(f, "%ld %ld", &age, &idle);
+	fclose(f);
+	if (ret != 2)
 		return 0;
 	return idle;
 }
@@ -1844,6 +1857,7 @@ static off_t get_procfile_size(const char *which)
 	while ((sz = getline(&line, &len, f)) != -1)
 		answer += sz;
 	fclose (f);
+	free(line);
 
 	return answer;
 }
